package net.wanji.opt.service.impl;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.gts.GeometryUtil;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.opt.bo.PolygonBO;
import net.wanji.opt.service.CrossManageService;
import net.wanji.opt.vo.CrossIdAndNameVO;
import org.locationtech.jts.geom.Geometry;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Kent HAN
 * @date 2023/6/25 8:56
 */
@Service
@Slf4j
public class CrossManageServiceImpl implements CrossManageService {
    private final BaseCrossInfoMapper crossInfoMapper;

    public CrossManageServiceImpl(BaseCrossInfoMapper crossInfoMapper) {
        this.crossInfoMapper = crossInfoMapper;
    }

    @Override
    public List<CrossIdAndNameVO> selectCrossesByPolygon(PolygonBO polygonBO) {
        List<CrossIdAndNameVO> res = new ArrayList<>();
        String wkt = polygonBO.getWkt();
        List<BaseCrossInfoPO> crossList = crossInfoMapper.selectAll();

        for (BaseCrossInfoPO cross : crossList) {
            String location = cross.getLocation();
            double[] lonLat = CrossUtil.getLonLat(location);
            String crossLonLat = lonLat[0] + "," + lonLat[1];
            Geometry wktGeo = GeometryUtil.str2Geometry(wkt);
            Geometry crossGeo = GeometryUtil.str2Geometry(crossLonLat);
            // 判断点和多边形是否相交
            boolean isIntersect = wktGeo.intersects(crossGeo);
            if (isIntersect) {
                CrossIdAndNameVO crossIdAndNameVO = new CrossIdAndNameVO();
                crossIdAndNameVO.setCrossId(cross.getId());
                crossIdAndNameVO.setCrossName(cross.getName());
                res.add(crossIdAndNameVO);
            }
        }

        return res;
    }
}
